var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { mod, params, repeatArray } from "./utils";
export const CONTEXT_TRACK_WINDOW = 5;
export const SPOTIFY_API = "https://api.spotify.com/v1";
export const config = {
    api: `${process.env.SPOTIFY_API || SPOTIFY_API}`,
};
export const transformContextTrack = (payload) => {
    var _a;
    const { id, name, uri, artists, album } = payload;
    return {
        artists,
        name,
        uri,
        id: id || "",
        position: null,
        album: {
            name: album.name,
            image: {
                url: (_a = album.images[0]) === null || _a === void 0 ? void 0 : _a.url
            }
        }
    };
};
// API Requests
export const getDevices = () => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const res = yield fetch(`${config.api}/me/player/devices`, {
            credentials: "include"
        }).then((res) => res.json());
        if (res.error) {
            if (res.error.status === 401) {
            }
            return [];
        }
        return res.devices;
    }
    catch (e) {
        console.error("Failed to fetch devices", e);
        return [];
    }
});
export const getContextItem = (url) => __awaiter(void 0, void 0, void 0, function* () {
    const proxy = url.replace(SPOTIFY_API, config.api);
    try {
        const res = yield fetch(proxy, {
            credentials: "include"
        }).then((res) => res.json());
        if (res.error) {
            if (res.error.status === 401) {
            }
            return null;
        }
        return res;
    }
    catch (e) {
        console.error("Failed to fetch context item", e);
        return null;
    }
});
export const createTrackContextFromApi = (state) => __awaiter(void 0, void 0, void 0, function* () {
    const { context, item: current_track } = state;
    let length = 0;
    let name = "";
    let type = "unknown";
    let id = null;
    let position = 0;
    let uri = null;
    let url = null;
    let prev = [];
    let next = [];
    if (context) {
        ({ type, uri, href: url } = context);
        type = context.uri.split(":")[1];
        id = context.uri.split(":")[2];
        if (type === "album" || type === "playlist") {
            let next_tracks = [];
            let previous_tracks = [];
            const item = yield getContextItem(url);
            if (item && item.tracks) {
                const tracks = type === "playlist"
                    ? item.tracks.items.map((item) => item.track)
                    : item.tracks.items;
                name = item.name;
                length = tracks.length;
                position = tracks.findIndex((child) => child.id === current_track.id);
                if (position === -1) {
                    position = null;
                    prev = [];
                    next = [];
                }
                else {
                    // Get next and previous tracks
                    const iter = Math.ceil(CONTEXT_TRACK_WINDOW / length);
                    const trackloop = repeatArray(tracks, iter * 2 + 1);
                    const looppos = position + length;
                    next_tracks = trackloop.slice(looppos + 1, looppos + 1 + CONTEXT_TRACK_WINDOW);
                    previous_tracks = trackloop.slice(looppos - CONTEXT_TRACK_WINDOW, looppos);
                    next = next_tracks.map((track, i) => (Object.assign(Object.assign({}, transformContextTrack(track)), { position: mod(position + (i + 1), length) })));
                    prev = previous_tracks.map((track, i) => (Object.assign(Object.assign({}, transformContextTrack(track)), { position: mod(position - (CONTEXT_TRACK_WINDOW - (i + 1)) - 1, length) })));
                }
            }
        }
    }
    return {
        id,
        name,
        length,
        type,
        uri,
        url,
        prev,
        next,
        current: Object.assign(Object.assign({}, transformContextTrack(current_track)), { // TODO(aelsen)
            position }),
    };
});
export const createTrackContextFromPlayer = (state) => __awaiter(void 0, void 0, void 0, function* () {
    const { context, track_window } = state;
    const { metadata, uri } = context;
    const { name } = metadata;
    const { current_track, next_tracks, previous_tracks } = track_window;
    const id = uri ? uri.split(":")[2] : null;
    const type = uri ? uri.split(":")[1] : "unknown";
    const url = `https://api.spotify.com/v1/${type}s/${id}`;
    let position = 0;
    let length = 0;
    if (type === "album" || type === "playlist") {
        const item = yield getContextItem(url);
        if (item === null || item === void 0 ? void 0 : item.tracks) {
            length = item.tracks.total || 0;
            position = item.tracks.items.findIndex((child) => {
                const track = type === "album" ? child : child.track;
                return track.id === current_track.id;
            });
        }
    }
    return {
        id,
        name,
        length,
        type,
        uri,
        url,
        current: Object.assign(Object.assign({}, transformContextTrack(current_track)), { position }),
        next: next_tracks.map((track, i) => (Object.assign(Object.assign({}, transformContextTrack(track)), { position: mod(position + (i + 1), length) }))),
        prev: previous_tracks.map((track, i) => (Object.assign(Object.assign({}, transformContextTrack(track)), { position: mod(position - (i + 1), length) })))
    };
});
export const getPlayerState = () => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    try {
        const res = yield fetch(`${config.api}/me/player`, {
            credentials: "include",
            method: "GET"
        }).then((res) => res.status === 200 ? res.json() : null);
        if (!res)
            return null;
        if (res.error) {
            if (((_a = res === null || res === void 0 ? void 0 : res.error) === null || _a === void 0 ? void 0 : _a.status) === 401) {
            }
            return null;
        }
        return res;
    }
    catch (e) {
        console.error("Failed to get state", e);
        return null;
    }
});
export const setPlayerPlayContext = (context, deviceId) => __awaiter(void 0, void 0, void 0, function* () {
    const payload = {
        position_ms: context.position,
    };
    if (context.uri)
        payload.context_uri = context.uri;
    if (context.offset)
        payload.offset = context.offset;
    fetch(`${config.api}/me/player/play${params({ device_id: deviceId })}`, {
        credentials: "include",
        method: 'PUT',
        body: JSON.stringify(payload)
    });
});
export const setPlayerPause = (deviceId) => __awaiter(void 0, void 0, void 0, function* () {
    fetch(`${config.api}/me/player/pause${params({ device_id: deviceId })}`, {
        credentials: "include",
        method: 'PUT'
    });
});
export const setPlayerPosition = (position, deviceId) => __awaiter(void 0, void 0, void 0, function* () {
    return fetch(`${config.api}/me/player/seek${params({ position_ms: position, device_id: deviceId })}`, {
        credentials: "include",
        method: 'PUT',
    });
});
export const setPlayerRepeat = (state, deviceId) => __awaiter(void 0, void 0, void 0, function* () {
    return fetch(`${config.api}/me/player/repeat${params({ state, device_id: deviceId })}`, {
        credentials: "include",
        method: 'PUT',
    });
});
export const setPlayerShuffle = (shuffle, deviceId) => __awaiter(void 0, void 0, void 0, function* () {
    return fetch(`${config.api}/me/player/shuffle${params({ shuffle, device_id: deviceId })}`, {
        credentials: "include",
        method: 'PUT',
    });
});
export const setPlayerVolume = (value, deviceId) => __awaiter(void 0, void 0, void 0, function* () {
    return fetch(`${config.api}/me/player/volume${params({ volume_percent: value, device_id: deviceId })}`, {
        credentials: "include",
        method: "PUT"
    });
});
export const setPlayerDevice = (deviceId) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const res = yield fetch(`${config.api}/me/player`, {
            credentials: "include",
            method: "PUT",
            body: JSON.stringify({ device_ids: [deviceId] })
        }).then((res) => res.status === 200 ? res.json() : null);
        if (!res)
            return;
        if (res.error) {
            if (res.error.status === 401) {
            }
            return;
        }
    }
    catch (e) {
        console.error("Failed to transfer playback", e);
    }
});
