var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { useCallback, useEffect, useRef, useState } from "react";
import { useInterval } from "./useInterval";
import { CONTEXT_TRACK_WINDOW, createTrackContextFromApi, createTrackContextFromPlayer, getDevices, getPlayerState, setPlayerDevice, setPlayerPause, setPlayerPlayContext, setPlayerPosition, setPlayerRepeat, setPlayerShuffle, setPlayerVolume, transformContextTrack, } from "./rest";
import { RepeatState } from "./types";
import { mod } from "./utils";
const POLL_PERIOD = 1000; // ms
const initialState = {
    disabled: true,
    duration: 600,
    local: true,
    paused: true,
    position: 0,
    repeat: RepeatState.OFF,
    shuffle: false,
    volume: 50,
};
;
export const useSpotifyPlayer = ({ token, pollPeriod = POLL_PERIOD, onContextChanged }) => {
    const contextRef = useRef(null);
    const [devices, setDevices] = useState([]);
    const [deviceId, setDeviceId] = useState(null);
    const [playerState, setPlayerState] = useState(initialState);
    const playerRef = useRef(null);
    const { startInterval, stopInterval } = useInterval();
    const player = playerRef.current;
    const { disabled, duration, local, paused, position, repeat, shuffle, volume } = playerState;
    const getAvailableDevices = useCallback(() => __awaiter(void 0, void 0, void 0, function* () {
        const devices = yield getDevices();
        setDevices(devices);
    }), []);
    const transferPlayback = (deviceId) => __awaiter(void 0, void 0, void 0, function* () {
        setPlayerDevice(deviceId);
    });
    const updatePlayerPositionByInterval = useCallback(() => {
        if (!playerRef.current) {
            stopInterval("seek");
            return;
        }
        setPlayerState((prev) => (Object.assign(Object.assign({}, prev), { position: prev.position + POLL_PERIOD })));
    }, [stopInterval]);
    const handleRepeat = () => __awaiter(void 0, void 0, void 0, function* () {
        const next = (playerState.repeat + 1) % 3;
        const state = RepeatState[next].toLowerCase();
        yield setPlayerRepeat(state);
        setPlayerState((prev) => (Object.assign(Object.assign({}, prev), { repeat: next })));
    });
    const handleShuffle = () => __awaiter(void 0, void 0, void 0, function* () {
        yield setPlayerShuffle(!shuffle);
        setPlayerState((prev) => (Object.assign(Object.assign({}, prev), { shuffle: !shuffle })));
    });
    const handleSeek = (position) => {
        if (local)
            player === null || player === void 0 ? void 0 : player.seek(position);
        else
            setPlayerPosition(position);
    };
    const handlePlay = (play) => {
        if (play) {
            if (local)
                player === null || player === void 0 ? void 0 : player.resume();
            else
                setPlayerPlayContext({ position });
        }
        else {
            if (local)
                player === null || player === void 0 ? void 0 : player.pause();
            else
                setPlayerPause();
        }
    };
    const handleNextTrack = () => {
        const context = contextRef.current;
        const restart = repeat === RepeatState.TRACK;
        if (restart) {
            if (local) {
                player.seek(0);
            }
            else {
                setPlayerPosition(0);
                setPlayerState((prev) => (Object.assign(Object.assign({}, prev), { position: 0 })));
            }
        }
        else {
            if (local) {
                player.nextTrack();
            }
            else {
                if (!context)
                    return;
                const offset = {};
                const next = context.next[0];
                const newContext = Object.assign(Object.assign({}, context), { prev: [context.current, ...context.prev], next: context.next.slice(1), current: next });
                if (next) {
                    offset.position = next.position !== null ? next.position : next.uri;
                }
                else {
                    offset.position = mod(context.current.position + 1, context.length);
                }
                setPlayerPlayContext({ position: 0, uri: context.uri, offset });
                setPlayerState((prev) => (Object.assign(Object.assign({}, prev), { position: 0 })));
                onContextChanged === null || onContextChanged === void 0 ? void 0 : onContextChanged(newContext);
                contextRef.current = newContext;
            }
        }
        ;
    };
    const handlePreviousTrack = () => __awaiter(void 0, void 0, void 0, function* () {
        const context = contextRef.current;
        const restart = repeat === RepeatState.TRACK || position > 5000;
        if (restart) {
            if (local) {
                player.seek(0);
            }
            else {
                setPlayerPosition(0);
                setPlayerState((prev) => (Object.assign(Object.assign({}, prev), { position: 0 })));
            }
        }
        else {
            if (local) {
                player.previousTrack();
            }
            else {
                if (!context)
                    return;
                const offset = {};
                const prev = context.prev[context.prev.length - 1];
                const newContext = Object.assign(Object.assign({}, context), { prev: context.prev.slice(0, -1), next: [context.current, ...context.next], current: prev });
                if (prev) {
                    offset.position = prev.position !== null ? prev.position : prev.uri;
                }
                else {
                    offset.position = mod(context.current.position - 1, context.length);
                }
                // console.log("Spotify Playback | Handle Prev Track --- using offset", offset, "setting next context", newContext);
                setPlayerPlayContext({ position: 0, uri: context.uri, offset });
                setPlayerState((prev) => (Object.assign(Object.assign({}, prev), { position: 0 })));
                contextRef.current = newContext;
            }
        }
        ;
    });
    const handleVolumeChange = (volume) => __awaiter(void 0, void 0, void 0, function* () {
        yield setPlayerVolume(volume);
        setPlayerState((prev) => (Object.assign(Object.assign({}, prev), { volume })));
    });
    const handleState = useCallback((state) => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        const { context: ctx, device, is_playing, item, progress_ms: position, repeat_state, shuffle_state: shuffle } = state;
        const context = contextRef.current;
        const { volume_percent: volume } = device;
        const duration = (item === null || item === void 0 ? void 0 : item.duration_ms) || 0;
        const paused = !is_playing;
        const repeat = RepeatState[repeat_state.toUpperCase()];
        if (!paused)
            startInterval(updatePlayerPositionByInterval, POLL_PERIOD, "seek");
        if (paused)
            stopInterval("seek");
        const newTrack = item
            ? transformContextTrack(item)
            : null;
        // Check if context has changed
        const contextChanged = (ctx === null || ctx === void 0 ? void 0 : ctx.uri) !== (context === null || context === void 0 ? void 0 : context.uri);
        const trackChanged = (newTrack === null || newTrack === void 0 ? void 0 : newTrack.uri) !== ((_a = context === null || context === void 0 ? void 0 : context.current) === null || _a === void 0 ? void 0 : _a.uri);
        const positionChanged = context && (context.next.length < CONTEXT_TRACK_WINDOW || context.prev.length < CONTEXT_TRACK_WINDOW);
        const stale = contextChanged || trackChanged || positionChanged;
        let newContext;
        if (stale) {
            newContext = yield createTrackContextFromApi(state);
            contextRef.current = newContext;
            onContextChanged === null || onContextChanged === void 0 ? void 0 : onContextChanged(newContext);
        }
        // console.log("Spotify Playback | Handle State, Fetched - setting next context", stale, newContext);
        // console.log("Spotify Playback | Handle State, Fetched --- context changed?", contextChanged, ctx?.uri, context?.uri);
        // console.log("Spotify Playback | Handle State, Fetched --- track changed?", trackChanged, newTrack?.uri, context?.current?.uri);
        setPlayerState((prev) => (Object.assign(Object.assign({}, prev), { position: position !== null ? position : prev.position, volume: volume !== null ? volume : prev.volume, duration,
            paused,
            repeat,
            shuffle })));
    }), [onContextChanged, startInterval, stopInterval, updatePlayerPositionByInterval]);
    const handlePlayerStateChanged = useCallback((state) => __awaiter(void 0, void 0, void 0, function* () {
        var _b, _c;
        if (!state) {
            setPlayerState((prev) => (Object.assign(Object.assign({}, prev), { local: false })));
            return;
        }
        const { context: ctx, duration, position, paused, repeat_mode: repeat, shuffle, track_window } = state;
        const context = contextRef.current;
        const volume = ((yield ((_b = playerRef.current) === null || _b === void 0 ? void 0 : _b.getVolume())) || 0) * 100;
        const contextTrackUri = (_c = track_window.current_track) === null || _c === void 0 ? void 0 : _c.uri;
        if (!paused)
            startInterval(updatePlayerPositionByInterval, POLL_PERIOD, "seek");
        if (paused)
            stopInterval("seek");
        // Check if context has changed
        const contextChanged = ctx.uri !== (context === null || context === void 0 ? void 0 : context.uri);
        const trackChanged = contextTrackUri !== (context === null || context === void 0 ? void 0 : context.current.uri);
        let newContext;
        if (contextChanged || trackChanged) {
            newContext = yield createTrackContextFromPlayer(state);
            contextRef.current = newContext;
        }
        setPlayerState((prev) => (Object.assign(Object.assign({}, prev), { duration,
            paused,
            position,
            repeat,
            shuffle,
            volume, local: true })));
    }), [startInterval, stopInterval, updatePlayerPositionByInterval]);
    const getState = useCallback(() => __awaiter(void 0, void 0, void 0, function* () {
        const state = yield getPlayerState();
        if (!state)
            return;
        handleState(state);
    }), [handleState]);
    const initPlayer = useCallback(() => {
        if (!token)
            return;
        const script = document.createElement("script");
        script.src = "https://sdk.scdn.co/spotify-player.js";
        script.async = true;
        document.body.appendChild(script);
        window.onSpotifyWebPlaybackSDKReady = () => {
            const player = new window.Spotify.Player({
                name: "constellation.fm",
                getOAuthToken: (cb) => { cb(token); },
                volume: 0.5
            });
            playerRef.current = player;
            player.activateElement();
            player.addListener('ready', (device) => {
                console.log('Spotify Playback | Ready with Device ID', device);
                setDeviceId(device.device_id);
                setPlayerState((prev) => (Object.assign(Object.assign({}, prev), { disabled: false })));
                getState();
            });
            player.addListener('not_ready', () => {
                setDeviceId(null);
                setPlayerState((prev) => (Object.assign(Object.assign({}, prev), { disabled: true })));
            });
            player.addListener('initialization_error', ({ message }) => {
                console.error(message);
            });
            player.addListener('authentication_error', ({ message }) => {
                console.error(message);
            });
            player.addListener('account_error', ({ message }) => {
                console.error(message);
            });
            player.addListener('player_state_changed', (handlePlayerStateChanged));
            player.connect();
        };
    }, [token, getState, handlePlayerStateChanged]);
    const handleDevices = useCallback(() => {
        var _a;
        const local = !!((_a = devices.find(({ id }) => id === deviceId)) === null || _a === void 0 ? void 0 : _a.is_active);
        setPlayerState((prev) => (Object.assign(Object.assign({}, prev), { local })));
    }, [deviceId, devices]);
    const handleDismount = useCallback(() => {
        return stopInterval;
    }, [stopInterval]);
    const handlePlaybackSubscription = useCallback(() => {
        if (!local) {
            startInterval(getState, POLL_PERIOD, "state");
            return;
        }
        stopInterval("state");
    }, [getState, local, startInterval, stopInterval]);
    useEffect(() => {
        initPlayer();
        return () => {
        };
    }, [initPlayer, token, pollPeriod, handlePlayerStateChanged]);
    useEffect(handleDevices, [handleDevices]);
    useEffect(handleDismount, [handleDismount]);
    useEffect(handlePlaybackSubscription, [handlePlaybackSubscription]);
    return {
        context: contextRef.current,
        deviceId,
        devices,
        disabled,
        duration,
        local,
        paused,
        position,
        repeat,
        shuffle,
        volume,
        handleNextTrack,
        handlePlay,
        handlePreviousTrack,
        handleRepeat,
        handleSeek,
        handleShuffle,
        handleVolumeChange,
        getAvailableDevices,
        transferPlayback
    };
};
